package com.company.jmixdatagridflowdemo.view.customer;

import com.company.jmixdatagridflowdemo.entity.Customer;
import com.company.jmixdatagridflowdemo.entity.CustomerContact;
import com.company.jmixdatagridflowdemo.view.main.MainView;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.editor.Editor;
import com.vaadin.flow.router.Route;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.model.CollectionPropertyContainer;
import io.jmix.flowui.model.DataContext;
import io.jmix.flowui.view.*;

import com.vaadin.flow.component.button.ButtonVariant;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.textfield.EmailField;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.data.binder.Binder;
import com.vaadin.flow.data.validator.EmailValidator;

import java.util.List;

@Route(value = "customers/:id", layout = MainView.class)
@ViewController("grid_Customer.detail")
@ViewDescriptor("customer-detail-view.xml")
@EditedEntityContainer("customerDc")
public class CustomerDetailView extends StandardDetailView<Customer> {
    @ViewComponent
    private CollectionPropertyContainer<CustomerContact> customerContactDc;
    @ViewComponent
    private DataContext dataContext;

    @Subscribe("customerContactTable.create")
    public void onCustomerContactTableCreate(ActionPerformedEvent event) {
        CustomerContact customerContact = dataContext.create(CustomerContact.class);
        customerContact.setCustomer(getEditedEntity());
        customerContactDc.getMutableItems().add(customerContact);
    }

    @Subscribe
    public void onInit(InitEvent event) {
        GridBufferedInlineEditor();
    }


    public void GridBufferedInlineEditor() {
        ValidationMessage firstNameValidationMessage = new ValidationMessage();
        ValidationMessage lastNameValidationMessage = new ValidationMessage();
        ValidationMessage emailValidationMessage = new ValidationMessage();

        Grid<CustomerContact> grid = new Grid<>(CustomerContact.class, false);
        Editor<CustomerContact> editor = grid.getEditor();

        Grid.Column<CustomerContact> firstNameColumn = grid
                .addColumn(CustomerContact::getFirstName).setHeader("First name")
                .setWidth("120px").setFlexGrow(0);
        Grid.Column<CustomerContact> lastNameColumn = grid.addColumn(CustomerContact::getLastName)
                .setHeader("Last name").setWidth("120px").setFlexGrow(0);
        Grid.Column<CustomerContact> emailColumn = grid.addColumn(CustomerContact::getEmail)
                .setHeader("Email");
        Grid.Column<CustomerContact> editColumn = grid.addComponentColumn(person -> {
            Button editButton = new Button("Edit");
            editButton.addClickListener(e -> {
                if (editor.isOpen())
                    editor.cancel();
                grid.getEditor().editItem(person);
            });
            return editButton;
        }).setWidth("150px").setFlexGrow(0);

        Binder<CustomerContact> binder = new Binder<>(CustomerContact.class);
        editor.setBinder(binder);
        editor.setBuffered(true);

        TextField firstNameField = new TextField();
        firstNameField.setWidthFull();
        binder.forField(firstNameField)
                .asRequired("First name must not be empty")
                .withStatusLabel(firstNameValidationMessage)
                .bind(CustomerContact::getFirstName, CustomerContact::setFirstName);
        firstNameColumn.setEditorComponent(firstNameField);

        TextField lastNameField = new TextField();
        lastNameField.setWidthFull();
        binder.forField(lastNameField).asRequired("Last name must not be empty")
                .withStatusLabel(lastNameValidationMessage)
                .bind(CustomerContact::getLastName, CustomerContact::setLastName);
        lastNameColumn.setEditorComponent(lastNameField);

        EmailField emailField = new EmailField();
        emailField.setWidthFull();
        binder.forField(emailField).asRequired("Email must not be empty")
                .withValidator(new EmailValidator(
                        "Enter a valid email address"))
                .withStatusLabel(emailValidationMessage)
                .bind(CustomerContact::getEmail, CustomerContact::setEmail);
        emailColumn.setEditorComponent(emailField);

        Button saveButton = new Button("Save", e -> editor.save());
        Button cancelButton = new Button(VaadinIcon.CLOSE.create(),
                e -> editor.cancel());
        cancelButton.addThemeVariants(ButtonVariant.LUMO_ICON,
                ButtonVariant.LUMO_ERROR);
        HorizontalLayout actions = new HorizontalLayout(saveButton,
                cancelButton);
        actions.setPadding(false);
        editColumn.setEditorComponent(actions);

        editor.addCancelListener(e -> {
            firstNameValidationMessage.setText("");
            lastNameValidationMessage.setText("");
            emailValidationMessage.setText("");
        });

      //  List<CustomerContact> people = DataService.getPeople();
      //  grid.setItems(people);

        getThemeList().clear();
        getThemeList().add("spacing-s");
        add(grid, firstNameValidationMessage, lastNameValidationMessage,
                emailValidationMessage);
    }

}